package cz.kebrt.unitcheck.console;

import java.io.PrintStream;

import org.apache.log4j.Logger;

import cz.kebrt.unitcheck.api.UnitCheckListener;
import cz.kebrt.unitcheck.api.data.CheckResult;
import cz.kebrt.unitcheck.api.data.CheckRunResult;
import cz.kebrt.unitcheck.api.data.ExceptionFailure;
import cz.kebrt.unitcheck.api.data.PathHolder;
import cz.kebrt.unitcheck.api.data.PropertyFailure;
import cz.kebrt.unitcheck.api.data.Success;
import cz.kebrt.unitcheck.api.data.TestMethod;
import cz.kebrt.unitcheck.config.PathPrinterConfig;
import cz.kebrt.unitcheck.jpf.PathPrinter;

/**
 * Prints UnitCheck results in the specified stream.
 *
 * @author michal
 */
public class ConsoleUnitCheckListener implements UnitCheckListener {

    private static final int SECTION_SEPARATOR_LENGTH = 100;

    private static final Logger log = Logger.getLogger(ConsoleUnitCheckListener.class);

    private final PrintStream out;
    private final PathPrinterConfig pathPrinterConfig;
    private final boolean printPath;

    /**
     * Constructs an instance.
     *
     * @param out stream to be used for printing results
     * @param pathPrinterConfig configuration used when printing path
     * @param printPath print error path or not
     */
    public ConsoleUnitCheckListener(PrintStream out, PathPrinterConfig pathPrinterConfig,
            boolean printPath) {
        this.out = out;
        this.pathPrinterConfig = pathPrinterConfig;
        this.printPath = printPath;
    }

    @Override
    public void checkRunStarted(TestMethod testMethod) {
        log.trace(String.format("checkRunStarted: method=%s.%s;",
                testMethod.getClassName(), testMethod.getMethodName()));

        printSectionStart("CHECKRUN STARTED");
        out.println(String.format("  test method=%s.%s", testMethod.getClassName(),
                testMethod.getMethodName()));
        printSectionEnd();
    }

    @Override
    public void checkRunFinished(TestMethod testMethod, CheckRunResult result) {
        log.trace(String.format(
                "checkRunFinished: method=%s.%s; checkCount=%d; exceptionFailures=%d",
                testMethod.getClassName(), testMethod.getMethodName(),
                result.getChecksCount(), result.getFailuresCount()));

        printSectionStart("CHECKRUN FINISHED");
        out.println(String.format("  test method=%s.%s",
                testMethod.getClassName(), testMethod.getMethodName()));
        out.println(String.format("  checks count=%d", result.getChecksCount()));
        out.println(String.format("  successes count=%d",
                result.getChecksCount() - result.getFailuresCount()));
        out.println(String.format("  failures count=%d", result.getFailuresCount()));
        printSectionEnd();
    }

    @Override
    public void checkFinished(TestMethod testMethod, CheckResult result) {
        log.trace(String.format("checkFinished: method=%s.%s; result=%s",
                testMethod.getClassName(), testMethod.getMethodName(),
                result.isSuccessful()));

        printSectionStart("CHECK FINISHED");
        if (result instanceof ExceptionFailure) {
            printExceptionFailure((ExceptionFailure) result);
        } else if (result instanceof PropertyFailure) {
            printPropertyFailure((PropertyFailure) result);
        } else if (result instanceof Success) {
            out.println("  success");
        } else {
            out.println("  unknown result");
        }
        printSectionEnd();
    }

    @Override
    public void checkRunInterrupted(TestMethod tm, CheckRunResult result) {
        printSectionStart("CHECKRUN INTERRUPTED");
        out.println(String.format("  test method=%s.%s",
                tm.getClassName(), tm.getMethodName()));
        printSectionEnd();
    }

    // ========================================================================

    private void printExceptionFailure(ExceptionFailure f) {
        log.trace(f.getMessage());
        out.println(String.format("  exception: %s", f.getException().getMessage()));
        f.getException().printStackTrace(out);
        if (printPath) {
            out.println("  path:");
            printPath(f.getPath());
        }
    }

    private void printPropertyFailure(PropertyFailure f) {
        out.println(String.format("  property violation: %s", f.getDescription()
                .getAsString()));
        out.println("  snapshot:");
        out.println(f.getSnapShot());
        if (printPath) {
            out.println("  path:");
            printPath(f.getPath());
        }
    }

    private void printPath(PathHolder path) {
        PathPrinter printer = new PathPrinter(pathPrinterConfig);
        printer.printPath(path, out);
    }

    private void printSectionStart(String name) {
        out.println();
        out.print("======== ");
        out.print(name);
        out.print(" ");
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < SECTION_SEPARATOR_LENGTH - name.length(); ++i) {
            sb.append('=');
        }
        out.println(sb.toString());
    }

    private void printSectionEnd() {
        out.println();
    }

}
