package cz.kebrt.unitcheck.junit;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import junit.runner.Version;

import org.junit.internal.JUnitSystem;
import org.junit.internal.RealSystem;
import org.junit.runner.Description;
import org.junit.runner.JUnitCore;
import org.junit.runner.Request;
import org.junit.runner.Result;
import org.junit.runner.notification.Failure;

/**
 * JUnit runner that allows to run single test methods. It is used
 * as a replacement for the default JUnitCore runner which all tests
 * within test classes.
 * <p>
 * Input test classes/methods are specified in the following form:
 * <ul>
 *   <li><code>class-name#method-name</code> - only a specified method will be executed
 *   <li><code>class-name</code> - all test methods will be executed
 * </ul>
 * @author michal
 */
public final class SingleTestRunner {

    protected static final String METHOD_DELIMITER = "#";

    /**
     * Default JUnit runner.
     */
    private JUnitCore core = new JUnitCore();

    private SingleTestRunner() { }

    /**
     * Executes the input test methods/classes. {@link SingleTestFilter} is used
     * for filtering methods that are not to be executed. {@link TestReportListener}
     * observes the execution.
     *
     * @param args list of method/class names
     */
    public static void main(String[] args) {
        JUnitSystem system = new RealSystem();
        SingleTestRunner runner = new SingleTestRunner();

        Result result = runner.run(system, args);
        system.exit(result.wasSuccessful() ? 0 : 1);
    }

    private Result run(JUnitSystem system, String... args) {
        system.out().println("JUnit version " + Version.id());

        List<Class<?>> classes = new ArrayList<Class<?>>();
        Set<String> classesFound = new HashSet<String>();
        List<Failure> missingClasses = new ArrayList<Failure>();
        SingleTestFilter filter = new SingleTestFilter();

        for (String each : args) {
            try {
                String clsName = getClassName(each);
                if (!classesFound.contains(clsName)) {
                    classes.add(Class.forName(clsName));
                    classesFound.add(clsName);
                }
                filter.addAllowed(each);
            } catch (ClassNotFoundException e) {
                system.out().println("Could not find class: " + each);
                Description description = Description.createSuiteDescription(each);
                Failure failure = new Failure(description, e);
                missingClasses.add(failure);
            }
        }

        core.addListener(new TestReportListener());
        Request req = Request.classes(classes.toArray(new Class[0]));
        req = req.filterWith(filter);
        Result result = core.run(req);

        for (Failure each : missingClasses) {
            result.getFailures().add(each);
        }
        return result;
    }

    private String getClassName(String arg) {
        if (arg.contains(METHOD_DELIMITER)) {
            String[] split = arg.split(METHOD_DELIMITER);
            return split[0];
        }
        return arg;
    }

}
